@@ -227,6 +227,24 @@ def req_guest_get_console_output(start_index, *args, **kwargs):
     return url, body
 
 
+def req_guest_live_migrate(start_index, *args, **kwargs):
+    url = '/guests/%s/action'
+    body = {'action': 'live_migrate_vm',
+            'dest_zcc_userid': args[start_index],
+            'destination': args[start_index + 1],
+            'parms': args[start_index + 2],
+            'operation': args[start_index + 3]}
+    return url, body
+
+
+def req_guest_pre_migrate(start_index, *args, **kwargs):
+    url = '/guests/%s/action'
+    body = {'action': 'register_vm',
+            'meta': args[start_index],
+            'net_set': args[start_index + 1]}
+    return url, body
+
+
 def req_guest_live_resize_cpus(start_index, *args, **kwargs):
     url = '/guests/%s/action'
     body = {'action': 'live_resize_cpus',
@@ -241,6 +259,20 @@ def req_guest_resize_cpus(start_index, *args, **kwargs):
     return url, body
 
 
+def req_guest_resize_mem(start_index, *args, **kwargs):
+    url = '/guests/%s/action'
+    body = {'action': 'resize_mem',
+            'size': args[start_index]}
+    return url, body
+
+
+def req_guest_live_resize_mem(start_index, *args, **kwargs):
+    url = '/guests/%s/action'
+    body = {'action': 'live_resize_mem',
+            'size': args[start_index]}
+    return url, body
+
+
 def req_guest_capture(start_index, *args, **kwargs):
     url = '/guests/%s/action'
     body = {'action': 'capture',
@@ -333,25 +365,21 @@ def req_guest_config_minidisks(start_index, *args, **kwargs):
     return url, body
 
 
-# FIXME: (userid, os_type) in one params, how to parse?
 def req_volume_attach(start_index, *args, **kwargs):
-    url = '/guests/%s/volumes'
-    body = {'info': {'os_type': args[start_index],
-                     'volume': args[start_index + 1],
-                     'connection': args[start_index + 2],
-                     'rollback': args[start_index + 3]}}
-    fill_kwargs_in_body(body['info'], **kwargs)
+    url = '/guests/volumes'
+    body = {'info': {'connection': args[start_index]}}
     return url, body
 
 
-# FIXME: (userid, os_type) in one params, how to parse?
 def req_volume_detach(start_index, *args, **kwargs):
-    url = '/guests/%s/volumes'
-    body = {'info': {'os_type': args[start_index],
-                     'volume': args[start_index + 1],
-                     'connection': args[start_index + 2],
-                     'rollback': args[start_index + 3]}}
-    fill_kwargs_in_body(body['info'], **kwargs)
+    url = '/guests/volumes'
+    body = {'info': {'connection': args[start_index]}}
+    return url, body
+
+
+def req_get_volume_connector(start_index, *args, **kwargs):
+    url = '/volumes/conn/%s'
+    body = None
     return url, body
 
 
@@ -422,7 +450,7 @@ def req_file_import(start_index, *args, **kwargs):
 
 def req_file_export(start_index, *args, **kwargs):
     url = '/files'
-    body = args[start_index]
+    body = {'source_file': args[start_index]}
 
     return url, body
 
@@ -567,6 +595,16 @@ DATABASE = {
         'args_required': 1,
         'params_path': 1,
         'request': req_guest_get_console_output},
+    'guest_register': {
+        'method': 'POST',
+        'args_required': 3,
+        'params_path': 1,
+        'request': req_guest_pre_migrate},
+    'guest_live_migrate': {
+        'method': 'POST',
+        'args_required': 5,
+        'params_path': 1,
+        'request': req_guest_live_migrate},
     'guest_live_resize_cpus': {
         'method': 'POST',
         'args_required': 2,
@@ -577,6 +615,16 @@ DATABASE = {
         'args_required': 2,
         'params_path': 1,
         'request': req_guest_resize_cpus},
+    'guest_live_resize_mem': {
+        'method': 'POST',
+        'args_required': 2,
+        'params_path': 1,
+        'request': req_guest_live_resize_mem},
+    'guest_resize_mem': {
+        'method': 'POST',
+        'args_required': 2,
+        'params_path': 1,
+        'request': req_guest_resize_mem},
     'guest_capture': {
         'method': 'POST',
         'args_required': 2,
@@ -644,14 +692,19 @@ DATABASE = {
         'request': req_guest_config_minidisks},
     'volume_attach': {
         'method': 'POST',
-        'args_required': 3,
-        'params_path': 1,
+        'args_required': 1,
+        'params_path': 0,
         'request': req_volume_attach},
     'volume_detach': {
         'method': 'DELETE',
-        'args_required': 3,
-        'params_path': 1,
+        'args_required': 1,
+        'params_path': 0,
         'request': req_volume_detach},
+    'get_volume_connector': {
+        'method': 'GET',
+        'args_required': 1,
+        'params_path': 1,
+        'request': req_get_volume_connector},
     'host_get_info': {
         'method': 'GET',
         'args_required': 0,
@@ -749,7 +802,7 @@ class RESTClient(object):
 
     def __init__(self, ip='127.0.0.1', port=8888,
                  ssl_enabled=False, verify=False,
-                 token_path='/etc/zvmsdk/token.dat'):
+                 token_path=None):
         # SSL enable or not
         if ssl_enabled:
             self.base_url = "https://" + ip + ":" + str(port)
@@ -818,7 +871,7 @@ class RESTClient(object):
         func = DATABASE[api_name]['request']
         url, body = func(count_params_in_path, *args, **kwargs)
 
-        if api_name in ['file_import', 'file_export']:
+        if api_name in ['file_import']:
             headers['Content-Type'] = 'application/octet-stream'
 
         if count_params_in_path > 0:
@@ -860,19 +913,21 @@ class RESTClient(object):
             except TypeError:
                 # if data is a file-like object
                 body = body
-        _headers['X-Auth-Token'] = self._get_token()
+
+        if self.token_path is not None:
+            _headers['X-Auth-Token'] = self._get_token()
 
         content_type = headers['Content-Type']
         stream = content_type == 'application/octet-stream'
         if stream:
             response = requests.request(method, url, data=body,
-                                    headers=_headers,
-                                    verify=self.verify,
-                                    stream=stream)
+                                        headers=_headers,
+                                        verify=self.verify,
+                                        stream=stream)
         else:
             response = requests.request(method, url, data=body,
-                                    headers=_headers,
-                                    verify=self.verify)
+                                        headers=_headers,
+                                        verify=self.verify)
         return response
 
     def call(self, api_name, *args, **kwargs):
