@@ -405,7 +405,9 @@ running SDK is able to read write and execute the directory.
         default='',
         opt_type='str',
         help='''
-volume fcp list
+volume fcp list.
+
+SDK will only use the fcp devices in the scope of this value.
 '''),
     # tests options
     Opt('images',
@@ -433,6 +435,14 @@ volume fcp list
     Opt('restapi_url',
         section='tests',
         default='http://127.0.0.1:8888'),
+    Opt('zvm_fcp',
+        section='tests'),
+    Opt('target_wwpn',
+        section='tests'),
+    Opt('target_lun',
+        section='tests'),
+    Opt('mount_point',
+        section='tests'),
     ]
 
 
