@@ -12,8 +12,9 @@
 #    under the License.
 
 
-import json
 import os
+import subprocess
+import tempfile
 import time
 import unittest
 
@@ -22,7 +23,8 @@ from parameterized import parameterized
 from zvmsdk.tests.fvt import base
 from zvmsdk.tests.fvt import test_utils
 from zvmsdk import config
-
+from zvmsdk import utils
+from zvmsdk import database
 
 CONF = config.CONF
 TEST_IMAGE_LIST = test_utils.TEST_IMAGE_LIST
@@ -59,22 +61,33 @@ class GuestHandlerBase(base.ZVMConnectorBaseTestCase):
         self.userid = self.utils.generate_test_userid()
         self.test_vsw = "RESTVSW1"
         self.test_vsw2 = "RESTVSW2"
+        self._GuestDbOperator = database.GuestDbOperator()
 
     def _check_CPU_MEM(self, userid, cpu_cnt=None, cpu_cnt_live=None,
+                       memory_size=None,
+                       memory_size_live=None,
+                       check_reserve=True,
                        maxcpu=CONF.zvm.user_default_max_cpu,
                        maxmem=CONF.zvm.user_default_max_memory):
         resp_info = self.client.guest_get_definition_info(userid)
         self.assertEqual(200, resp_info.status_code)
 
         Statement = "USER %s LBYONLY" % userid.upper()
-        resp_content = json.loads(resp_info.content)
+        resp_content = test_utils.load_output(resp_info.content)
         user_direct = resp_content['output']['user_direct']
         cpu_num = 0
         cpu_num_live = 0
         cpu_num_online = 0
+
         for ent in user_direct:
             if ent.startswith(Statement):
+                memory_direct = ent.split()[3].strip()
                 max_memory = ent.split()[4].strip()
+                memory_direct = int(utils.convert_to_mb(memory_direct))
+            if ent.startswith("COMMAND DEF STOR RESERVED"):
+                memory_reserve_direct = ent.split()[4].strip()
+                memory_reserve_direct = int(utils.convert_to_mb(
+                                                    memory_reserve_direct))
             if ent.startswith("CPU"):
                 cpu_num = cpu_num + 1
             if ent.startswith("MACHINE ESA"):
@@ -91,11 +104,26 @@ class GuestHandlerBase(base.ZVMConnectorBaseTestCase):
                 online_state = c.strip().upper()
                 if online_state == "Y":
                     cpu_num_online = cpu_num_online + 1
+        if memory_size_live is not None:
+            memory_info = self._smutclient.execute_cmd(userid, "lsmem")[-2:]
+            online_memory = int(memory_info[0].split()[4].strip())
+            offline_memory = int(memory_info[1].split()[3].strip())
+
         if cpu_cnt is not None:
             self.assertEqual(cpu_cnt, cpu_num)
+        if memory_size is not None:
+            memory_size = int(utils.convert_to_mb(memory_size))
+            self.assertEqual(memory_size, memory_direct)
         if cpu_cnt_live is not None:
             self.assertEqual(cpu_cnt_live, cpu_num_live)
             self.assertEqual(cpu_cnt_live, cpu_num_online)
+
+        if memory_size_live is not None:
+            memory_size_live = int(utils.convert_to_mb(memory_size_live))
+            self.assertEqual(memory_size_live, online_memory)
+            if check_reserve:
+                self.assertEqual(memory_reserve_direct, offline_memory)
+
         self.assertEqual(maxcpu, max_cpus)
         self.assertEqual(maxmem, max_memory)
 
@@ -201,7 +229,7 @@ class GuestHandlerBase(base.ZVMConnectorBaseTestCase):
 
         # Continue to check the nic info defined in vswitch table
         resp = self.client.guests_get_nic_info(userid)
-        nics_info = json.loads(resp.content)['output']
+        nics_info = test_utils.load_output(resp.content)['output']
 
         for nic in nics_info:
             if (nic['interface'] != vdev) or (nic['userid'] != userid):
@@ -216,12 +244,18 @@ class GuestHandlerBase(base.ZVMConnectorBaseTestCase):
         return True, None
 
     def _make_transport_file(self):
-        os.mkdir('openstack')
-        os.mkdir('openstack/latest')
-        transport_file = "openstack/latest/meta_data.json"
-        transport_file1 = "openstack/latest/network_data.json"
-        transport_file2 = "openstack/latest/vendor_data.json"
-        transport_file3 = "openstack/latest/vendor_data2.json"
+        tempDir = tempfile.mkdtemp()
+        os.chmod(tempDir, 0o777)
+        os.mkdir('/'.join([tempDir, 'openstack']))
+        os.mkdir('/'.join([tempDir, 'openstack/latest']))
+        transport_file = '/'.join([tempDir,
+                                "openstack/latest/meta_data.json"])
+        transport_file1 = '/'.join([tempDir,
+                            "openstack/latest/network_data.json"])
+        transport_file2 = '/'.join([tempDir,
+                            "openstack/latest/vendor_data.json"])
+        transport_file3 = '/'.join([tempDir,
+                            "openstack/latest/vendor_data2.json"])
         with open(transport_file, 'w') as f:
             f.write('{"admin_pass": "sMcTNh8b65dM",\
                       "random_seed": "Q2UzyJ+6ITjY4STr/sSkDeoP4Wy\
@@ -245,7 +279,7 @@ class GuestHandlerBase(base.ZVMConnectorBaseTestCase):
                       c+4O3LrQu1GOY1+JyC146EW951tsW0bKWigM=",\
                       "uuid": "d93b1ef2-5ca7-459f-beaf-eedea6b6ee43",\
                       "availability_zone": "nova",\
-                      "hostname": "deploy_tests",\
+                      "hostname": "deploy_fvt",\
                       "launch_index": 0,\
                       "devices"    : [],\
                       "project_id": "6e3ab252b982424c859fa83ff281e8ab",\
@@ -256,12 +290,35 @@ class GuestHandlerBase(base.ZVMConnectorBaseTestCase):
             f.write('{}')
         with open(transport_file3, 'w') as f:
             f.write('{}')
-        os.system('tar -czvf cfgdrive.tgz openstack')
-        os.system('cp cfgdrive.tgz /var/lib/zvmsdk/cfgdrive.tgz')
-        os.system('rm -rf openstack')
-        os.system('rm cfgdrive.tgz')
-        os.system('chown -R zvmsdk:zvmsdk /var/lib/zvmsdk/cfgdrive.tgz')
-        os.system('chmod -R 755 /var/lib/zvmsdk/cfgdrive.tgz')
+        cmd = ['mkisofs',
+               '-o',
+               '/var/lib/zvmsdk/cfgdrive.iso',
+               '-ldots',
+               '-allow-lowercase',
+               '-allow-multidot',
+               '-l',
+               '-quiet',
+               '-J',
+               '-r',
+               '-V',
+               'config-2',
+               tempDir]
+        try:
+            output = subprocess.check_output(cmd,
+                                             close_fds=True,
+                                             stderr=subprocess.STDOUT)
+            output = bytes.decode(output)
+            output.split()[2]
+        except subprocess.CalledProcessError as e:
+            msg = e.output
+            print ("Create cfgdrive.iso meet error: %s" % msg)
+        except Exception as e:
+            msg = e.output
+            print ("Create cfgdrive.iso meet error: %s" % msg)
+
+        os.system('rm -rf %s' % tempDir)
+        os.system('chown -R zvmsdk:zvmsdk /var/lib/zvmsdk/cfgdrive.iso')
+        os.system('chmod -R 755 /var/lib/zvmsdk/cfgdrive.iso')
 
     def _get_free_osa(self):
         osa_info = self._smutclient._query_OSA()
@@ -467,9 +524,9 @@ class GuestHandlerTestCase(GuestHandlerBase):
         self.assertTrue(self.utils.wait_until_create_userid_complete(userid))
 
         self._make_transport_file()
-        self.addCleanup(os.system, 'rm /var/lib/zvmsdk/cfgdrive.tgz')
+        self.addCleanup(os.system, 'rm /var/lib/zvmsdk/cfgdrive.iso')
 
-        transport_file = '/var/lib/zvmsdk/cfgdrive.tgz'
+        transport_file = '/var/lib/zvmsdk/cfgdrive.iso'
         image_name = self.utils.import_image_if_not_exist(image_path,
                                                           os_version)
 
@@ -477,7 +534,7 @@ class GuestHandlerTestCase(GuestHandlerBase):
                                         transportfiles=transport_file)
         self.assertEqual(200, resp.status_code)
 
-        # todo: create newwork interface
+        # todo: create network interface
         resp = self.client.guest_create_network_interface(userid, os_version,
                                                           guest_networks)
 
@@ -485,6 +542,11 @@ class GuestHandlerTestCase(GuestHandlerBase):
         self.assertEqual(200, resp.status_code)
         self.assertTrue(self.utils.wait_until_guest_reachable(userid))
 
+        # Verify cfgdrive.iso take effect
+        time.sleep(30)
+        result = self._smutclient.execute_cmd(userid, 'hostname')
+        self.assertIn('deploy_fvt', result)
+
         resp = self.client.guest_get_definition_info(userid)
         self.assertEqual(200, resp.status_code)
         self.apibase.verify_result('test_guest_get', resp.content)
@@ -515,9 +577,15 @@ class GuestHandlerTestCase(GuestHandlerBase):
         self.apibase.verify_result('test_guests_get_stats',
                                    resp.content)
 
+        # pause unpause a stopped instance should return 409
+        resp = self.client.guest_pause(userid)
+        self.assertEqual(409, resp.status_code)
+        resp = self.client.guest_unpause(userid)
+        self.assertEqual(409, resp.status_code)
+
         # Capture a powered off instance will lead to error
         resp = self.client.guest_capture(userid, 'failed')
-        self.assertEqual(500, resp.status_code)
+        self.assertEqual(409, resp.status_code)
 
         resp = self.client.guest_start(userid)
         self.assertEqual(200, resp.status_code)
@@ -525,7 +593,8 @@ class GuestHandlerTestCase(GuestHandlerBase):
 
         resp_info = self.client.guest_get_info(userid)
         self.assertEqual(200, resp_info.status_code)
-        resp_content = json.loads(resp_info.content)
+
+        resp_content = test_utils.load_output(resp_info.content)
         info_off = resp_content['output']
         self.assertEqual('on', info_off['power_state'])
         self.assertNotEqual(info_off['cpu_time_us'], 0)
@@ -544,9 +613,43 @@ class GuestHandlerTestCase(GuestHandlerBase):
 
         resp_state = self.client.guest_get_power_state(userid)
         self.assertEqual(200, resp_state.status_code)
-        resp_content = json.loads(resp_state.content)
+
+        resp_content = test_utils.load_output(resp_state.content)
         self.assertEqual('off', resp_content['output'])
 
+    @parameterized.expand(TEST_IMAGE_LIST)
+    def test_guest_create_deploy_set_hostname(self, case_name, image_path,
+                                              os_version):
+        """Deploy with hostname specified."""
+        userid = self._get_userid_auto_cleanup()
+        ip_addr = self.utils.generate_test_ip()
+        guest_networks = [{'ip_addr': ip_addr, 'cidr': CONF.tests.cidr}]
+        hostname = 'fakehostname'
+
+        resp = self.client.guest_create(userid)
+        self.assertEqual(200, resp.status_code)
+        self.assertTrue(self.utils.wait_until_create_userid_complete(userid))
+
+        image_name = self.utils.import_image_if_not_exist(image_path,
+                                                          os_version)
+
+        resp = self.client.guest_deploy(userid, image_name=image_name,
+                                        hostname=hostname)
+        self.assertEqual(200, resp.status_code)
+
+        # todo: create network interface
+        resp = self.client.guest_create_network_interface(userid, os_version,
+                                                          guest_networks)
+
+        resp = self.client.guest_start(userid)
+        self.assertEqual(200, resp.status_code)
+        self.assertTrue(self.utils.wait_until_guest_reachable(userid))
+
+        # Verify hostname changed
+        time.sleep(30)
+        result = self._smutclient.execute_cmd(userid, 'hostname')
+        self.assertIn('fakehostname', result)
+
     def test_guests_get_nic_info(self):
         resp = self.client.guests_get_nic_info()
         self.assertEqual(200, resp.status_code)
@@ -795,6 +898,125 @@ class GuestHandlerTestCase(GuestHandlerBase):
         self._check_CPU_MEM(userid, cpu_cnt=7, cpu_cnt_live=7,
                             maxcpu=10, maxmem="2048M")
 
+    @parameterized.expand(TEST_IMAGE_LIST)
+    def test_guest_live_resize_memory(self, case_name,
+                                    image_path, os_version):
+        userid = self._get_userid_auto_cleanup()
+        resp = self.client.guest_create(userid, memory=2048,
+                                        max_cpu=10, max_mem="10G")
+        self.assertEqual(200, resp.status_code)
+
+        image_name = self.utils.import_image_if_not_exist(image_path,
+                                                          os_version)
+
+        resp = self.client.guest_deploy(userid, image_name)
+        self.assertEqual(200, resp.status_code)
+
+        # Live resize a guest's CPU when the guest is not active
+        resp = self.client.guest_live_resize_memory(userid, "1G")
+        self.assertEqual(409, resp.status_code)
+
+        # Power on the guest
+        resp = self.client.guest_start(userid)
+        self.assertEqual(200, resp.status_code)
+        self.assertTrue(self.utils.wait_until_guest_reachable(userid))
+
+        # check cpu memory
+        self._check_CPU_MEM(userid, memory_size="2048M",
+                            memory_size_live="2048M", maxcpu=10, maxmem="10G")
+        self._check_total_memory(userid, maxmem="10G")
+
+        # Live resize a guest's memory when the guest is active, unit M
+        resp = self.client.guest_live_resize_memory(userid, "4096M")
+        self.assertEqual(200, resp.status_code)
+        self._check_CPU_MEM(userid, memory_size="4096M",
+                            memory_size_live="4096M", maxcpu=10, maxmem="10G")
+
+        # Live resize a guest's memory more than the current size, unit G
+        resp = self.client.guest_live_resize_memory(userid, "9G")
+        self.assertEqual(200, resp.status_code)
+        self._check_CPU_MEM(userid, memory_size="9G",
+                            memory_size_live="9G", maxcpu=10, maxmem="10G")
+
+        # Live resize a guest's memory less than the current size
+        resp = self.client.guest_live_resize_memory(userid, "4096M")
+        self.assertEqual(409, resp.status_code)
+        self._check_CPU_MEM(userid, memory_size="9G",
+                            memory_size_live="9G", maxcpu=10, maxmem="10G")
+
+        # Live resize a guest's memory equal to the current size
+        resp = self.client.guest_live_resize_memory(userid, "9G")
+        self.assertEqual(200, resp.status_code)
+        self._check_CPU_MEM(userid, memory_size="9G",
+                            memory_size_live="9G", maxcpu=10, maxmem="10G")
+
+        # Live resize a guest's memory more than the max size
+        resp = self.client.guest_live_resize_memory(userid, "20G")
+        self.assertEqual(409, resp.status_code)
+        self._check_CPU_MEM(userid, memory_size="9G",
+                            memory_size_live="9G", maxcpu=10, maxmem="10G")
+
+        # Resize a guest's memory more than the current size
+        resp = self.client.guest_resize_memory(userid, "10G")
+        self.assertEqual(200, resp.status_code)
+        self._check_CPU_MEM(userid, memory_size="10G",
+                            memory_size_live="9G", check_reserve=False,
+                            maxcpu=10, maxmem="10G")
+
+        # Resize a guest's memory less than the current size
+        resp = self.client.guest_resize_memory(userid, "8192M")
+        self.assertEqual(200, resp.status_code)
+        self._check_CPU_MEM(userid, memory_size="8192M",
+                            memory_size_live="9G", check_reserve=False,
+                            maxcpu=10, maxmem="10G")
+
+        # Resize a guest's memory equal to the current size
+        resp = self.client.guest_resize_memory(userid, "8192M")
+        self.assertEqual(200, resp.status_code)
+        self._check_CPU_MEM(userid, memory_size="8192M",
+                            memory_size_live="9G", check_reserve=False,
+                            maxcpu=10, maxmem="10G")
+
+        # Resize a guest's memory more than the max size
+        resp = self.client.guest_resize_memory(userid, "20G")
+        self.assertEqual(409, resp.status_code)
+        self._check_CPU_MEM(userid, memory_size="8192M",
+                            memory_size_live="9G", check_reserve=False,
+                            maxcpu=10, maxmem="10G")
+
+        # Restart guest, check memory
+        resp = self.client.guest_softstop(userid)
+        self.assertEqual(200, resp.status_code)
+        self.assertTrue(self.utils.wait_until_guest_in_power_state(userid,
+                                                                  "off"))
+        resp = self.client.guest_start(userid)
+        self.assertEqual(200, resp.status_code)
+        self.assertTrue(self.utils.wait_until_guest_reachable(userid))
+        self._check_CPU_MEM(userid, memory_size="8192M",
+                            memory_size_live="8192M", maxcpu=10, maxmem="10G")
+
+        # Live resize a guest's memory equal to the max size
+        resp = self.client.guest_live_resize_memory(userid, "10G")
+        self.assertEqual(200, resp.status_code)
+        self._check_CPU_MEM(userid, memory_size="10G",
+                            memory_size_live="10G", maxcpu=10, maxmem="10G")
+
+        # Live resize a guest's memory, invalid size/unit
+        resp = self.client.guest_live_resize_memory(userid, "1T")
+        self.assertEqual(400, resp.status_code)
+        self._check_CPU_MEM(userid, memory_size="10G",
+                            memory_size_live="10G", maxcpu=10, maxmem="10G")
+
+        resp = self.client.guest_live_resize_memory(userid, "10240m")
+        self.assertEqual(400, resp.status_code)
+        self._check_CPU_MEM(userid, memory_size="10G",
+                            memory_size_live="10G", maxcpu=10, maxmem="10G")
+
+        resp = self.client.guest_live_resize_memory(userid, "1024k")
+        self.assertEqual(400, resp.status_code)
+        self._check_CPU_MEM(userid, memory_size="10G",
+                            memory_size_live="10G", maxcpu=10, maxmem="10G")
+
 
 class GuestHandlerTestCaseWithCreatedGuest(GuestHandlerBase):
     """This class is used to test functions that requires a guest to be
@@ -896,7 +1118,8 @@ class GuestHandlerTestCaseWithSingleDeployedGuest(GuestHandlerBase):
         resp = self.client.guests_get_nic_info(self.userid_exists,
                                               nic_id='123456')
         self.assertEqual(200, resp.status_code)
-        nic_info = json.loads(resp.content)['output']
+
+        nic_info = test_utils.load_output(resp.content)['output']
         self.assertEqual("1006", nic_info[0]["interface"])
         self.assertEqual("123456", nic_info[0]["port"])
 
@@ -1150,6 +1373,28 @@ class GuestHandlerTestCaseWithMultipleDeployedGuest(GuestHandlerBase):
 
         GuestHandlerBase.tearDown(self)
 
+    @parameterized.expand(TEST_USERID_LIST)
+    def test_guest_live_migrate(self, case_name, userid, version):
+        # Move a running guest from one z/VM to another within an SSI cluster
+        # power on
+        self.client.guest_start(userid)
+        self.assertTrue(self.utils.wait_until_guest_in_power_state(
+                                                userid, "on"))
+        # live migrate
+        resp_test1 = self.client.guest_live_migrate_vm(userid, "LEON0001",
+                                         "opnstk2", {}, "test")
+
+        if resp_test1.status_code == 200:
+            resp = self.client.guest_live_migrate_vm(userid, "LEON0001",
+                                         "opnstk2", {}, "move")
+            self.assertEqual(200, resp.status_code)
+
+        comments = self._GuestDbOperator.get_comments_by_userid(userid)
+        comments['migrated'] = 0
+        self._GuestDbOperator.update_guest_by_userid(userid,
+                                                    comments=comments)
+        self.client.guest_delete(userid)
+
     @parameterized.expand(TEST_USERID_LIST)
     def test_guest_get_console_output(self, case_name, userid, version):
         # power on
@@ -1160,7 +1405,7 @@ class GuestHandlerTestCaseWithMultipleDeployedGuest(GuestHandlerBase):
         # get console output
         resp = self.client.guest_get_console_output(userid)
         self.assertEqual(200, resp.status_code)
-        outputs = json.loads(resp.content)['output']
+        outputs = test_utils.load_output(resp.content)['output']
         self.assertNotEqual(outputs, '')
 
     @parameterized.expand(TEST_USERID_LIST)
